/**************************************************************************
* iptrk - ip tracker
* by Jason Nunn <jsno@arafura.net.au>, Darwin, NT, Australia. Dec 2000
* For Arafura Connect
*
* Arafura Connect
* 71 McMinn St
* Darwin NT 0810
* Australia
*
* http://www.arafuraconnect.com.au
* http://jsno.arafuraconnect.com.au
*
* iptrk is a commercial product. usage is unauthorised without written
* aggreement.
*
* =======================================================================
* user space process that tracks /dev/iptrk
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"
#include "iptrk.p"

PRIVATE int  parent_pid;
PRIVATE char *tty_name;

/**************************************************************************
* exception routines
**************************************************************************/
PRIVATE void atend_h(int retv, void *arg)
{
  if(parent_pid != getpid())  /*if not parent terminating then ignore*/
    return;

  log_m('I', NULL, " -- Ending iptrk --");
  log_m('I', NULL, " switching off iptrk code");

  if(iptrk_fp != NULL)
  {
    putc('0', iptrk_fp);
    fflush(iptrk_fp);
    close_device();
  }

  promiscuous_mode(0);
  save_treaps();
  log_m('I', NULL, " bye");

  if(curses_f)
  {
    clear();
    refresh();
    curses_deinit();
  }

  if(tty_name != NULL)
  {
    printf("iptrk ended.. Waiting before respawning..\n");
    fflush(stdout);
    sleep(5);
  }
}

PRIVATE void reaper_h(int sig)
{
  log_m('I', NULL, " exiting");
  exit(-1);
}

PRIVATE void seg_fault_h(int sig)
{
  log_m('I', NULL, " segmentation fault. Aborting");
  exit(-1);
}

PRIVATE void user_h(int sig)
{
  log_m('I', NULL, " user sig intercepted");
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void create_directories(void)
{
  int  i;
  struct passwd *p;
  char str[STR_LEN + 1],
       *dir_str[] =
  {
    IPTRK_PATH,
    IPTRK_DB_WATCH_PATH,
    IPTRK_DB_WATCH_PATHC,
    IPTRK_LOG_IP_PATH,
    IPTRK_LOG_IP_PATHC,
    IPTRK_GRAPHS_PATH,
    IPTRK_LOG_PORT_PATH,
    NULL
  };

  for(i = 0; dir_str[i] != NULL; i++)
  {
    if(!mkdir(dir_str[i], 0755))
    {
      sprintf(str, " created directory %s", dir_str[i]);
      log_m('I', NULL, str);
    }
  }

  p = getpwnam(WEB_OWNER);
  if(p != NULL)
  {
    if(!chown(IPTRK_GRAPHS_PATH, p->pw_uid, p->pw_gid))
    {
      sprintf(str, " changed ownership of %s to %s",
        IPTRK_GRAPHS_PATH, WEB_OWNER);
      log_m('I', NULL, str);
    }
  }
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void init(void)
{
  char str[STR_LEN + 1];
  struct passwd *pw;

  if(tally_processes("(iptrk)") > 1)
  {
    printf("Error: process already running. aborting.\n");
    exit(-1);
  }

  pw = getpwuid(getuid());
  if(pw != NULL)
  {
    if(strcmp(pw->pw_name, "root"))
    {
      printf("Error: Must be root to run iptrk.\n");
      exit(-1);
    }
  }

  timer = time(NULL);
  time_st = localtime(&timer);
  memcpy(&old_time_st, time_st, sizeof(struct tm));

  parent_pid = getpid();
  on_exit(atend_h, NULL);
  signal(SIGHUP, reaper_h);
  signal(SIGINT, reaper_h);
  signal(SIGTERM, reaper_h);
  signal(SIGHUP, reaper_h);
  signal(SIGPIPE, SIG_IGN);
  signal(SIGSEGV, seg_fault_h);
  signal(SIGUSR1, user_h);

  log_m('I', NULL, " -- Starting iptrk --");
  check_expiry_date();

  if(gethostname(our_hostname, STR_LEN) == -1)
  {
    log_e(__func__, "::gethostname()");
    exit(-1);
  }

  read_boot_conf();
  sprintf(str, " wait delay is %d us", boot.wait_delay);
  log_m('I', NULL, str);
  sprintf(str, " watch time is %d s", boot.watch_time);
  log_m('I', NULL, str);
  if(boot.cancel_gram)
    log_m('I', NULL, " packet cancellation is on");
  create_directories();

  log_m('I', NULL, " switching on iptrk code");
  promiscuous_mode(1);

  open_device();
  putc('1', iptrk_fp);
  fflush(iptrk_fp);
}

/**************************************************************************
*
**************************************************************************/
int main(int argc, char *argv[])
{
  int  x;

  welcome();

  for(x = 0; x < argc; x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          printf(
            "Usage: iptrk <options>\n"
             " -h       Display this help info\n"
             " -t <tty> If running from init, then define tty\n"
             "\n"
          );
          return 0;

        case 't':
          tty_name = parse_cmdline(argc, argv, x);
          break;
      }
    }
  }

  log_m = log_m_file;

  if(tty_name != NULL)
    attach_tty(tty_name);

  init();
  init_curses();
  rt_track();

  return 0;
}
