/**************************************************************************
* iptrkg- log grapher (iptrk - ip tracker)
* by Jason Nunn <jsno@arafura.net.au>, Aug 2002
* For Arafura Connect
*
* Arafura Connect
* 71 McMinn St
* Darwin NT 0810
* Australia
*
* http://www.arafuraconnect.com.au
* http://jsno.arafuraconnect.com.au
*
* iptrk is a commercial product. usage is unauthorised without written
* aggreement.
*
* =======================================================================
* produces time domain stats from log_ip logs
* first produced to product daily volume stats for morgan buckley
*
**************************************************************************/
#include "defs.h"
#include "lib.p"

PRIVATE char    ip_log_path[STR_LEN + 1] = IPTRK_LOG_IP_PATHC;
PRIVATE voldr_t total;

/**************************************************************************
*
**************************************************************************/
PRIVATE void print_daily_volumes(void)
{
  FILE    *fp;
  char    str[STR_LEN + 1],
          str1[STR_LEN + 1],
          str2[STR_LEN + 1],
          str3[STR_LEN + 1],
          str4[STR_LEN + 1],
          str5[STR_LEN + 1],
          date[STR_SMALL + 1],
          old_date[STR_SMALL + 1] = "",
          *s;
  voldr_t v;
  U_64    x;

  void print_ent(void)
  {
    print_vol_u64(str1, v.I);
    print_vol_u64(str2, v.O);
    print_vol_u64(str3, v.u);
    print_vol_u64(str4, v.i);
    print_vol_u64(str5, v.o);
    printf("%20s %20s %20s %20s %20s\n",
      str1, str2, str3, str4, str5);
  }

 /*
  * start
  */
  printf("loading log- %s\n", ip_log_path);
  fp = fopen(ip_log_path, "r");
  if(fp == NULL)
  {
    log_e(__func__, "::fopen()");
    exit(-1);
  }

  printf(
    "--\n%-15s %20s %20s %20s %20s %20s\n"
    "--\n",
    "DAY", "I", "O", "u", "i", "o");

  memset(&v, 0, sizeof(voldr_t));
  for(;;)
  {
    if(feof(fp))
      break;

    if(fgets(str, STR_LEN, fp) == NULL)
      break;

    if(str[0] != 'o')
      continue;

    if(sscanf(str, "%*c %*s %s", date) != 1)
      continue;

    if(strcmp(date, old_date))
    {
      if(old_date[0] != 0)
      {
        print_ent();
        memset(&v, 0, sizeof(voldr_t));
      }
      strcpy(old_date, date);
      printf("%-15s ", date);
    }

    if(fgets(str, STR_LEN, fp) == NULL)
      break;

    s = str;
    if(!get_next_vol_u64(&s, &x)) continue;
    v.I += x;
    total.I += x;
    if(!get_next_vol_u64(&s, &x)) continue;
    v.O += x;
    total.O += x;
    if(!get_next_vol_u64(&s, &x)) continue;
    v.u += x;
    total.u += x;
    if(!get_next_vol_u64(&s, &x)) continue;
    v.i += x;
    total.i += x;
    if(!get_next_vol_u64(&s, &x)) continue;
    v.o += x;
    total.o += x;
  }
  fclose(fp);

  print_ent();
  print_vol_u64(str1, total.I);
  print_vol_u64(str2, total.O);
  print_vol_u64(str3, total.u);
  print_vol_u64(str4, total.i);
  print_vol_u64(str5, total.o);
  printf(
    "--\n"
    "%15s %20s %20s %20s %20s %20s\n",
    "", str1, str2, str3, str4, str5);
}

/**************************************************************************
*
**************************************************************************/
int main(int argc, char *argv[])
{
  int x;

  welcome();
  timer = time(NULL);
  for(x = 0; x < argc; x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          printf(
            "Usage: iptrkq_log_ip <options>\n"
             " -h                    Display this help info\n"
             " -f <ip_log directory> ip_log's to interrogate.\n"
             "\n"
          );
          return 0;

        case 'f':
          strncpy2(ip_log_path, parse_cmdline(argc, argv, x), STR_LEN);
          break;
      }
    }
  }

  print_daily_volumes();
  printf("--\nEND\n");

  return 0;
}
