/**************************************************************************
* iptrk - ip tracker
* by Jason Nunn <jsno@arafura.net.au>, Darwin, NT, Australia. Dec 2000
* For Arafura Connect
*
* Arafura Connect
* 71 McMinn St
* Darwin NT 0810
* Australia
*
* http://www.arafuraconnect.com.au
* http://jsno.arafuraconnect.com.au
*
* iptrk is a commercial product. usage is unauthorised without written
* aggreement.
*
* =======================================================================
* maintenance code (address dosa detection)
*
* this is a pretty simple detection mech. go through watch tree, and
* compare each source and destination instance with every other address in
* the tree.
*
**************************************************************************/
#include <curses.h>
#include "defs.h"
#include "lib.p"
#include "iptrk.p"

PRIVATE dosa_t  *dosa;
PRIVATE U_32    key_addr;
PRIVATE int     s_tally, d_tally;

/**************************************************************************
*
**************************************************************************/
PRIVATE void tally_tree(treap_t *t)
{
  iptrk_t *n;

  if(t == NULL)
    return;

  tally_tree(t->l);

  n = TREAP_ENTRY(t);
  if(key_addr == n->addr.addr.s)
    s_tally++;
  if(key_addr == n->addr.addr.d)
    d_tally++;

  tally_tree(t->r);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void test_addr(void)
{
  char           str[STR_LEN + 1],
                 email_body[STR_LEN_HUGE];
  struct in_addr in;

  void trig(char typ, int tally)
  {
    in.s_addr = htonl(key_addr);
    sprintf(str, " DoSA addr(%c) Trig %d- %d (%s: > %d)",
      typ, dosa->no, tally, inet_ntoa(in), dosa->limit);
    log_m('I', NULL, str);
    compose_watch_email_body(email_body, dosa->str, inet_ntoa(in));
    email_operator(str, email_body);
  }

 /*
  * start
  */
  s_tally = 0;
  d_tally = 0;
  tally_tree(watch_bt);
  if(s_tally >= dosa->limit)
    trig('S', s_tally);
  if(d_tally >= dosa->limit)
    trig('D', d_tally);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void walk_local_addrs(treap_t *t)
{
  if(t == NULL)
    return;

  walk_local_addrs(t->l);
  key_addr = *((U_32 *)TREAP_ENTRY(t));
  test_addr();
  walk_local_addrs(t->r);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void do_dosa_address(void)
{
  ll_t *ll;

  log_m('I', NULL, " doing DoSA detection (addr)");
  ll = boot.dosa_addr_ll;
  for(;;)
  {
    dosa = ll_read_entry(&ll);
    if(dosa == NULL)
      break;

    if(!test_zone(dosa->zone))
      continue;

    walk_local_addrs(our_net_ip_bt);
  }
}
